use std::{path::PathBuf, str::FromStr};

use log::info;
use tower_lsp::lsp_types::*;

use crate::{server::LSPServer, utils::{from_lsp_pos, to_escape_path, to_lsp_range}};

pub fn document_highlight(
    server: &LSPServer,
    params: &TextDocumentPositionParams
) -> Option<Vec<DocumentHighlight>> {
    let uri = &params.text_document.uri;
    let file_id = server.srcs.get_id(&uri).to_owned();
    server.srcs.wait_parse_ready(file_id, false);
    let projects = server.srcs.design_file_map.read().ok()?;

    let path = match PathBuf::from_str(uri.path()) {
        Ok(path) => path,
        Err(error) => {
            info!("error happen in <goto_include_definition>: {:?}", error);
            return None;
        }
    };
    let escape_path = to_escape_path(&path);

    let project = match projects.get("VHDL_Project") {
        Some(project) => project,
        None => return None
    };

    let source = project.get_source(&escape_path)?;

    let ent = project.find_declaration(&source, from_lsp_pos(params.position))?;

    Some(
        project
            .find_all_references_in_source(&source, ent)
            .iter()
            .map(|pos| DocumentHighlight {
                range: to_lsp_range(pos.range()),
                kind: Some(DocumentHighlightKind::TEXT),
            })
            .collect(),
    )
}