use log::info;
use tower_lsp::lsp_types::*;
use crate::{server::LSPServer, utils::to_lsp_range};
use crate::utils::{to_escape_path, to_symbol_kind};

use std::path::PathBuf;
use std::str::FromStr;
use vhdl_lang::{EntHierarchy, Token};

pub fn document_symbol(server: &LSPServer, params: &DocumentSymbolParams) -> Option<DocumentSymbolResponse> {
    let uri = &params.text_document.uri;
    let file_id = server.srcs.get_id(&uri).to_owned();
    server.srcs.wait_parse_ready(file_id, false);
    let projects = server.srcs.design_file_map.read().ok()?;

    let path = match PathBuf::from_str(uri.path()) {
        Ok(path) => path,
        Err(error) => {
            info!("error happen in <goto_include_definition>: {:?}", error);
            return None;
        }
    };
    let escape_path = to_escape_path(&path);

    let project = match projects.get("VHDL_Project") {
        Some(project) => project,
        None => return None
    };

    let source = project.get_source(&escape_path)?;

    // Some files are mapped to multiple libraries, only use the first library for document symbols
    let library_name = project
        .library_mapping_of(&source)
        .into_iter()
        .next()?;

    // TODO: Confirm vscode support
    // if server.client_has_hierarchical_document_symbol_support() {
        fn to_document_symbol(
            EntHierarchy { ent, children }: EntHierarchy,
            ctx: &Vec<Token>,
        ) -> DocumentSymbol {
            // Use the declaration position, if it exists,
            // else the position of the first source range token.
            // The latter is applicable for unnamed elements, e.g., processes or loops.
            let selection_pos = ent.decl_pos().unwrap_or(ent.src_span.start_token.pos(ctx));
            let src_range = ent.src_span.pos(ctx).range();
            #[allow(deprecated)]
            DocumentSymbol {
                name: ent.describe(),
                kind: to_symbol_kind(ent.kind()),
                tags: None,
                detail: None,
                selection_range: to_lsp_range(selection_pos.range),
                range: to_lsp_range(src_range),
                children: if !children.is_empty() {
                    Some(
                        children
                            .into_iter()
                            .map(|hierarchy| to_document_symbol(hierarchy, ctx))
                            .collect(),
                    )
                } else {
                    None
                },
                deprecated: None,
            }
        }

        Some(DocumentSymbolResponse::Nested(
            project
                .document_symbols(&library_name, &source)
                .into_iter()
                .map(|(hierarchy, tokens)| to_document_symbol(hierarchy, tokens))
                .collect(),
        ))
    // } else {
    //     #[allow(clippy::ptr_arg)]
    //     fn to_symbol_information(ent: EntRef, ctx: &Vec<Token>) -> SymbolInformation {
    //         let selection_pos = ent.decl_pos().unwrap_or(ent.src_span.start_token.pos(ctx));
    //         #[allow(deprecated)]
    //         SymbolInformation {
    //             name: ent.describe(),
    //             kind: to_symbol_kind(ent.kind()),
    //             tags: None,
    //             location: srcpos_to_location(selection_pos),
    //             deprecated: None,
    //             container_name: ent.parent_in_same_source().map(|ent| ent.describe()),
    //         }
    //     }

    //     Some(DocumentSymbolResponse::Flat(
    //         project
    //             .document_symbols(&library_name, &source)
    //             .into_iter()
    //             .flat_map(|(a, ctx)| {
    //                 a.into_flat()
    //                     .into_iter()
    //                     .map(|hierarchy| to_symbol_information(hierarchy, ctx))
    //             })
    //             .collect(),
    //     ))
    // }
}