use std::{path::PathBuf, str::FromStr};

use log::info;
use tower_lsp::lsp_types::*;
use crate::server::LSPServer;

use super::{from_lsp_pos, to_escape_path};

pub fn hover(server: &LSPServer, params: &HoverParams) -> Option<Hover> {
    let uri = &params.text_document_position_params.text_document.uri;
    let pos = params.text_document_position_params.position;
    let file_id = server.srcs.get_id(&uri).to_owned();
    server.srcs.wait_parse_ready(file_id, false);
    let projects = server.srcs.design_file_map.read().ok()?;

    let path = match PathBuf::from_str(uri.path()) {
        Ok(path) => path,
        Err(error) => {
            info!("error happen in <goto_include_definition>: {:?}", error);
            return None;
        }
    };
    let escape_path = to_escape_path(&path);

    let project = match projects.get("VHDL_Project") {
        Some(project) => project,
        None => return None
    };

    let source = project.get_source(&escape_path)?;

    let ent = project.find_declaration(&source, from_lsp_pos(pos))?;

    let value = project.format_declaration(ent)?;

    Some(Hover {
        contents: HoverContents::Markup(MarkupContent {
            kind: MarkupKind::Markdown,
            value: format!("```vhdl\n{value}\n```"),
        }),
        range: None,
    })
}